/* langinfo.c --- Windows language and locale information by katahiromz */
/* This file is public domain software (PDS). */
#include <windows.h>
#include <stdio.h>

#ifndef _countof
    #define _countof(array) sizeof(array) / sizeof(array[0])
#endif

void show_help(void)
{
    printf("langinfo --- Windows language and locale information\n");
    printf("OPTIONS:\n");
    printf("  --langid XXXX\n");
    printf("  --lcid XXXXXXXX\n");
}

typedef struct LANGID_ENTRY
{
    WORD Value;
    const char *Name;
} LANGID_ENTRY;

const LANGID_ENTRY LangIDEntries[] =
{
    { LANG_NEUTRAL, "LANG_NEUTRAL" },
    { LANG_AFRIKAANS, "LANG_AFRIKAANS" },
    { LANG_ALBANIAN, "LANG_ALBANIAN" },
    { LANG_ARABIC, "LANG_ARABIC" },
    { LANG_ARMENIAN, "LANG_ARMENIAN" },
    { LANG_ASSAMESE, "LANG_ASSAMESE" },
    { LANG_AZERI, "LANG_AZERI" },
    { LANG_BASQUE, "LANG_BASQUE" },
    { LANG_BELARUSIAN, "LANG_BELARUSIAN" },
    { LANG_BENGALI, "LANG_BENGALI" },
    { LANG_BULGARIAN, "LANG_BULGARIAN" },
    { LANG_CATALAN, "LANG_CATALAN" },
    { LANG_CHINESE, "LANG_CHINESE" },
    { LANG_CROATIAN, "LANG_CROATIAN" },
    { LANG_CZECH, "LANG_CZECH" },
    { LANG_DANISH, "LANG_DANISH" },
    { LANG_DUTCH, "LANG_DUTCH" },
    { LANG_ENGLISH, "LANG_ENGLISH" },
    { LANG_ESTONIAN, "LANG_ESTONIAN" },
    { LANG_FAEROESE, "LANG_FAEROESE" },
    { LANG_FARSI, "LANG_FARSI" },
    { LANG_FINNISH, "LANG_FINNISH" },
    { LANG_FRENCH, "LANG_FRENCH" },
    { LANG_GEORGIAN, "LANG_GEORGIAN" },
    { LANG_GERMAN, "LANG_GERMAN" },
    { LANG_GREEK, "LANG_GREEK" },
    { LANG_GUJARATI, "LANG_GUJARATI" },
    { LANG_HEBREW, "LANG_HEBREW" },
    { LANG_HINDI, "LANG_HINDI" },
    { LANG_HUNGARIAN, "LANG_HUNGARIAN" },
    { LANG_ICELANDIC, "LANG_ICELANDIC" },
    { LANG_INDONESIAN, "LANG_INDONESIAN" },
    { LANG_ITALIAN, "LANG_ITALIAN" },
    { LANG_JAPANESE, "LANG_JAPANESE" },
    { LANG_KANNADA, "LANG_KANNADA" },
    { LANG_KASHMIRI, "LANG_KASHMIRI" },
    { LANG_KAZAK, "LANG_KAZAK" },
    { LANG_KONKANI, "LANG_KONKANI" },
    { LANG_KOREAN, "LANG_KOREAN" },
    { LANG_LATVIAN, "LANG_LATVIAN" },
    { LANG_LITHUANIAN, "LANG_LITHUANIAN" },
    { LANG_MACEDONIAN, "LANG_MACEDONIAN" },
    { LANG_MALAY, "LANG_MALAY" },
    { LANG_MALAYALAM, "LANG_MALAYALAM" },
    { LANG_MANIPURI, "LANG_MANIPURI" },
    { LANG_MARATHI, "LANG_MARATHI" },
    { LANG_NEPALI, "LANG_NEPALI" },
    { LANG_NORWEGIAN, "LANG_NORWEGIAN" },
    { LANG_ORIYA, "LANG_ORIYA" },
    { LANG_POLISH, "LANG_POLISH" },
    { LANG_PORTUGUESE, "LANG_PORTUGUESE" },
    { LANG_PUNJABI, "LANG_PUNJABI" },
    { LANG_ROMANIAN, "LANG_ROMANIAN" },
    { LANG_RUSSIAN, "LANG_RUSSIAN" },
    { LANG_SANSKRIT, "LANG_SANSKRIT" },
    { LANG_SERBIAN, "LANG_SERBIAN" },
    { LANG_SINDHI, "LANG_SINDHI" },
    { LANG_SLOVAK, "LANG_SLOVAK" },
    { LANG_SLOVENIAN, "LANG_SLOVENIAN" },
    { LANG_SPANISH, "LANG_SPANISH" },
    { LANG_SWAHILI, "LANG_SWAHILI" },
    { LANG_SWEDISH, "LANG_SWEDISH" },
    { LANG_TAMIL, "LANG_TAMIL" },
    { LANG_TATAR, "LANG_TATAR" },
    { LANG_TELUGU, "LANG_TELUGU" },
    { LANG_THAI, "LANG_THAI" },
    { LANG_TURKISH, "LANG_TURKISH" },
    { LANG_UKRAINIAN, "LANG_UKRAINIAN" },
    { LANG_URDU, "LANG_URDU" },
    { LANG_UZBEK, "LANG_UZBEK" },
    { LANG_VIETNAMESE, "LANG_VIETNAMESE" },
    { LANG_SYSTEM_DEFAULT, "LANG_SYSTEM_DEFAULT" },
    { LANG_USER_DEFAULT, "LANG_USER_DEFAULT" },
};

typedef struct SUBLANGID_ENTRY
{
    WORD LangID;
    WORD SubLangID;
    const char *Name;
} SUBLANGID_ENTRY;

const SUBLANGID_ENTRY SubLangIDEntries[] =
{
    { LANG_ARABIC, SUBLANG_ARABIC_SAUDI_ARABIA, "SUBLANG_ARABIC_SAUDI_ARABIA" },
    { LANG_ARABIC, SUBLANG_ARABIC_IRAQ, "SUBLANG_ARABIC_IRAQ" },
    { LANG_ARABIC, SUBLANG_ARABIC_EGYPT, "SUBLANG_ARABIC_EGYPT" },
    { LANG_ARABIC, SUBLANG_ARABIC_LIBYA, "SUBLANG_ARABIC_LIBYA" },
    { LANG_ARABIC, SUBLANG_ARABIC_ALGERIA, "SUBLANG_ARABIC_ALGERIA" },
    { LANG_ARABIC, SUBLANG_ARABIC_MOROCCO, "SUBLANG_ARABIC_MOROCCO" },
    { LANG_ARABIC, SUBLANG_ARABIC_TUNISIA, "SUBLANG_ARABIC_TUNISIA" },
    { LANG_ARABIC, SUBLANG_ARABIC_OMAN, "SUBLANG_ARABIC_OMAN" },
    { LANG_ARABIC, SUBLANG_ARABIC_YEMEN, "SUBLANG_ARABIC_YEMEN" },
    { LANG_ARABIC, SUBLANG_ARABIC_SYRIA, "SUBLANG_ARABIC_SYRIA" },
    { LANG_ARABIC, SUBLANG_ARABIC_JORDAN, "SUBLANG_ARABIC_JORDAN" },
    { LANG_ARABIC, SUBLANG_ARABIC_LEBANON, "SUBLANG_ARABIC_LEBANON" },
    { LANG_ARABIC, SUBLANG_ARABIC_KUWAIT, "SUBLANG_ARABIC_KUWAIT" },
    { LANG_ARABIC, SUBLANG_ARABIC_UAE, "SUBLANG_ARABIC_UAE" },
    { LANG_ARABIC, SUBLANG_ARABIC_BAHRAIN, "SUBLANG_ARABIC_BAHRAIN" },
    { LANG_ARABIC, SUBLANG_ARABIC_QATAR, "SUBLANG_ARABIC_QATAR" },
    { LANG_AZERI, SUBLANG_AZERI_LATIN, "SUBLANG_AZERI_LATIN" },
    { LANG_AZERI, SUBLANG_AZERI_CYRILLIC, "SUBLANG_AZERI_CYRILLIC" },
    { LANG_CHINESE, SUBLANG_CHINESE_TRADITIONAL, "SUBLANG_CHINESE_TRADITIONAL" },
    { LANG_CHINESE, SUBLANG_CHINESE_SIMPLIFIED, "SUBLANG_CHINESE_SIMPLIFIED" },
    { LANG_CHINESE, SUBLANG_CHINESE_HONGKONG, "SUBLANG_CHINESE_HONGKONG" },
    { LANG_CHINESE, SUBLANG_CHINESE_SINGAPORE, "SUBLANG_CHINESE_SINGAPORE" },
    { LANG_CHINESE, SUBLANG_CHINESE_MACAU, "SUBLANG_CHINESE_MACAU" },
    { LANG_DUTCH, SUBLANG_DUTCH, "SUBLANG_DUTCH" },
    { LANG_DUTCH, SUBLANG_DUTCH_BELGIAN, "SUBLANG_DUTCH_BELGIAN" },
    { LANG_ENGLISH, SUBLANG_ENGLISH_US, "SUBLANG_ENGLISH_US" },
    { LANG_ENGLISH, SUBLANG_ENGLISH_UK, "SUBLANG_ENGLISH_UK" },
    { LANG_ENGLISH, SUBLANG_ENGLISH_AUS, "SUBLANG_ENGLISH_AUS" },
    { LANG_ENGLISH, SUBLANG_ENGLISH_CAN, "SUBLANG_ENGLISH_CAN" },
    { LANG_ENGLISH, SUBLANG_ENGLISH_NZ, "SUBLANG_ENGLISH_NZ" },
    { LANG_ENGLISH, SUBLANG_ENGLISH_EIRE, "SUBLANG_ENGLISH_EIRE" },
    { LANG_ENGLISH, SUBLANG_ENGLISH_SOUTH_AFRICA, "SUBLANG_ENGLISH_SOUTH_AFRICA" },
    { LANG_ENGLISH, SUBLANG_ENGLISH_JAMAICA, "SUBLANG_ENGLISH_JAMAICA" },
    { LANG_ENGLISH, SUBLANG_ENGLISH_CARIBBEAN, "SUBLANG_ENGLISH_CARIBBEAN" },
    { LANG_ENGLISH, SUBLANG_ENGLISH_BELIZE, "SUBLANG_ENGLISH_BELIZE" },
    { LANG_ENGLISH, SUBLANG_ENGLISH_TRINIDAD, "SUBLANG_ENGLISH_TRINIDAD" },
    { LANG_ENGLISH, SUBLANG_ENGLISH_ZIMBABWE, "SUBLANG_ENGLISH_ZIMBABWE" },
    { LANG_ENGLISH, SUBLANG_ENGLISH_PHILIPPINES, "SUBLANG_ENGLISH_PHILIPPINES" },
    { LANG_FRENCH, SUBLANG_FRENCH, "SUBLANG_FRENCH" },
    { LANG_FRENCH, SUBLANG_FRENCH_BELGIAN, "SUBLANG_FRENCH_BELGIAN" },
    { LANG_FRENCH, SUBLANG_FRENCH_CANADIAN, "SUBLANG_FRENCH_CANADIAN" },
    { LANG_FRENCH, SUBLANG_FRENCH_SWISS, "SUBLANG_FRENCH_SWISS" },
    { LANG_FRENCH, SUBLANG_FRENCH_LUXEMBOURG, "SUBLANG_FRENCH_LUXEMBOURG" },
    { LANG_FRENCH, SUBLANG_FRENCH_MONACO, "SUBLANG_FRENCH_MONACO" },
    { LANG_GERMAN, SUBLANG_GERMAN, "SUBLANG_GERMAN" },
    { LANG_GERMAN, SUBLANG_GERMAN_SWISS, "SUBLANG_GERMAN_SWISS" },
    { LANG_GERMAN, SUBLANG_GERMAN_AUSTRIAN, "SUBLANG_GERMAN_AUSTRIAN" },
    { LANG_GERMAN, SUBLANG_GERMAN_LUXEMBOURG, "SUBLANG_GERMAN_LUXEMBOURG" },
    { LANG_GERMAN, SUBLANG_GERMAN_LIECHTENSTEIN, "SUBLANG_GERMAN_LIECHTENSTEIN" },
    { LANG_ITALIAN, SUBLANG_ITALIAN, "SUBLANG_ITALIAN" },
    { LANG_ITALIAN, SUBLANG_ITALIAN_SWISS, "SUBLANG_ITALIAN_SWISS" },
    { LANG_KASHMIRI, SUBLANG_KASHMIRI_INDIA, "SUBLANG_KASHMIRI_INDIA" },
    { LANG_KOREAN, SUBLANG_KOREAN, "SUBLANG_KOREAN" },
    { LANG_LITHUANIAN, SUBLANG_LITHUANIAN, "SUBLANG_LITHUANIAN" },
    { LANG_MALAY, SUBLANG_MALAY_MALAYSIA, "SUBLANG_MALAY_MALAYSIA" },
    { LANG_MALAY, SUBLANG_MALAY_BRUNEI_DARUSSALAM, "SUBLANG_MALAY_BRUNEI_DARUSSALAM" },
    { LANG_NEPALI, SUBLANG_NEPALI_INDIA, "SUBLANG_NEPALI_INDIA" },
    { LANG_NORWEGIAN, SUBLANG_NORWEGIAN_BOKMAL, "SUBLANG_NORWEGIAN_BOKMAL" },
    { LANG_NORWEGIAN, SUBLANG_NORWEGIAN_NYNORSK, "SUBLANG_NORWEGIAN_NYNORSK" },
    { LANG_PORTUGUESE, SUBLANG_PORTUGUESE, "SUBLANG_PORTUGUESE" },
    { LANG_PORTUGUESE, SUBLANG_PORTUGUESE_BRAZILIAN, "SUBLANG_PORTUGUESE_BRAZILIAN" },
    { LANG_SERBIAN, SUBLANG_SERBIAN_LATIN, "SUBLANG_SERBIAN_LATIN" },
    { LANG_SERBIAN, SUBLANG_SERBIAN_CYRILLIC, "SUBLANG_SERBIAN_CYRILLIC" },
    { LANG_SPANISH, SUBLANG_SPANISH, "SUBLANG_SPANISH" },
    { LANG_SPANISH, SUBLANG_SPANISH_MEXICAN, "SUBLANG_SPANISH_MEXICAN" },
    { LANG_SPANISH, SUBLANG_SPANISH_MODERN, "SUBLANG_SPANISH_MODERN" },
    { LANG_SPANISH, SUBLANG_SPANISH_GUATEMALA, "SUBLANG_SPANISH_GUATEMALA" },
    { LANG_SPANISH, SUBLANG_SPANISH_COSTA_RICA, "SUBLANG_SPANISH_COSTA_RICA" },
    { LANG_SPANISH, SUBLANG_SPANISH_PANAMA, "SUBLANG_SPANISH_PANAMA" },
    { LANG_SPANISH, SUBLANG_SPANISH_DOMINICAN_REPUBLIC, "SUBLANG_SPANISH_DOMINICAN_REPUBLIC" },
    { LANG_SPANISH, SUBLANG_SPANISH_VENEZUELA, "SUBLANG_SPANISH_VENEZUELA" },
    { LANG_SPANISH, SUBLANG_SPANISH_COLOMBIA, "SUBLANG_SPANISH_COLOMBIA" },
    { LANG_SPANISH, SUBLANG_SPANISH_PERU, "SUBLANG_SPANISH_PERU" },
    { LANG_SPANISH, SUBLANG_SPANISH_ARGENTINA, "SUBLANG_SPANISH_ARGENTINA" },
    { LANG_SPANISH, SUBLANG_SPANISH_ECUADOR, "SUBLANG_SPANISH_ECUADOR" },
    { LANG_SPANISH, SUBLANG_SPANISH_CHILE, "SUBLANG_SPANISH_CHILE" },
    { LANG_SPANISH, SUBLANG_SPANISH_URUGUAY, "SUBLANG_SPANISH_URUGUAY" },
    { LANG_SPANISH, SUBLANG_SPANISH_PARAGUAY, "SUBLANG_SPANISH_PARAGUAY" },
    { LANG_SPANISH, SUBLANG_SPANISH_BOLIVIA, "SUBLANG_SPANISH_BOLIVIA" },
    { LANG_SPANISH, SUBLANG_SPANISH_EL_SALVADOR, "SUBLANG_SPANISH_EL_SALVADOR" },
    { LANG_SPANISH, SUBLANG_SPANISH_HONDURAS, "SUBLANG_SPANISH_HONDURAS" },
    { LANG_SPANISH, SUBLANG_SPANISH_NICARAGUA, "SUBLANG_SPANISH_NICARAGUA" },
    { LANG_SPANISH, SUBLANG_SPANISH_PUERTO_RICO, "SUBLANG_SPANISH_PUERTO_RICO" },
    { LANG_SWEDISH, SUBLANG_SWEDISH, "SUBLANG_SWEDISH" },
    { LANG_SWEDISH, SUBLANG_SWEDISH_FINLAND, "SUBLANG_SWEDISH_FINLAND" },
    { LANG_URDU, SUBLANG_URDU_PAKISTAN, "SUBLANG_URDU_PAKISTAN" },
    { LANG_URDU, SUBLANG_URDU_INDIA, "SUBLANG_URDU_INDIA" },
    { LANG_UZBEK, SUBLANG_UZBEK_LATIN, "SUBLANG_UZBEK_LATIN" },
    { LANG_UZBEK, SUBLANG_UZBEK_CYRILLIC, "SUBLANG_UZBEK_CYRILLIC" },
};

typedef struct SORTID_ENTRY
{
    WORD LangID;
    WORD SortID;
    const char *Name;
} SORTID_ENTRY;

const SORTID_ENTRY SortIDEntries[] =
{
    { LANG_JAPANESE, SORT_JAPANESE_XJIS, "SORT_JAPANESE_XJIS" },
    { LANG_JAPANESE, SORT_JAPANESE_UNICODE, "SORT_JAPANESE_UNICODE" },
    { LANG_CHINESE, SORT_CHINESE_BIG5, "SORT_CHINESE_BIG5" },
    { LANG_CHINESE, SORT_CHINESE_PRCP, "SORT_CHINESE_PRCP" },
    { LANG_CHINESE, SORT_CHINESE_UNICODE, "SORT_CHINESE_UNICODE" },
    { LANG_CHINESE, SORT_CHINESE_PRC, "SORT_CHINESE_PRC" },
    { LANG_CHINESE, SORT_CHINESE_BOPOMOFO, "SORT_CHINESE_BOPOMOFO" },
    { LANG_KOREAN, SORT_KOREAN_KSC, "SORT_KOREAN_KSC" },
    { LANG_KOREAN, SORT_KOREAN_UNICODE, "SORT_KOREAN_UNICODE" },
    { LANG_GERMAN, SORT_GERMAN_PHONE_BOOK, "SORT_GERMAN_PHONE_BOOK" },
    { LANG_HUNGARIAN, SORT_HUNGARIAN_DEFAULT, "SORT_HUNGARIAN_DEFAULT" },
    { LANG_HUNGARIAN, SORT_HUNGARIAN_TECHNICAL, "SORT_HUNGARIAN_TECHNICAL" },
    { LANG_GEORGIAN, SORT_GEORGIAN_TRADITIONAL, "SORT_GEORGIAN_TRADITIONAL" },
    { LANG_GEORGIAN, SORT_GEORGIAN_MODERN, "SORT_GEORGIAN_MODERN" },
};

LANGID parse_langid(const char *name)
{
    INT i, nCount = _countof(LangIDEntries);
    for (i = 0; i < nCount; ++i)
    {
        if (lstrcmpiA(LangIDEntries[i].Name, name) == 0)
        {
            return LangIDEntries[i].Value;
        }
    }
    return 0xFFFFFFFF;
}

WORD parse_sublangid(LANGID LangID, const char *name)
{
    INT i, nCount = _countof(SubLangIDEntries);
    for (i = 0; i < nCount; ++i)
    {
        if (LangID != SubLangIDEntries[i].LangID)
            continue;

        if (lstrcmpiA(SubLangIDEntries[i].Name, name) == 0)
        {
            return SubLangIDEntries[i].SubLangID;
        }
    }

    if (lstrcmpiA(name, "SUBLANG_NEUTRAL") == 0)
        return SUBLANG_NEUTRAL;
    if (lstrcmpiA(name, "SUBLANG_DEFAULT") == 0)
        return SUBLANG_DEFAULT;
    if (lstrcmpiA(name, "SUBLANG_SYS_DEFAULT") == 0)
        return SUBLANG_SYS_DEFAULT;
    return SUBLANG_NEUTRAL;
}

void check_langid(LANGID LangID)
{
    INT i, nCount;
    const char *LangIDName = NULL;
    const char *SubLangIDName = NULL;

    nCount = _countof(LangIDEntries);
    for (i = 0; i < nCount; ++i)
    {
        if (LangIDEntries[i].Value == PRIMARYLANGID(LangID))
        {
            LangIDName = LangIDEntries[i].Name;
            break;
        }
    }

    nCount = _countof(SubLangIDEntries);
    for (i = 0; i < nCount; ++i)
    {
        if (SubLangIDEntries[i].LangID != PRIMARYLANGID(LangID))
            continue;
        if (SubLangIDEntries[i].SubLangID != SUBLANGID(LangID))
            continue;

        SubLangIDName = SubLangIDEntries[i].Name;
        break;
    }
    if (SubLangIDName == NULL)
    {
        switch (SUBLANGID(LangID))
        {
            case SUBLANG_NEUTRAL:
                SubLangIDName = "SUBLANG_NEUTRAL";
                break;
            case SUBLANG_DEFAULT:
                SubLangIDName = "SUBLANG_DEFAULT";
                break;
            case SUBLANG_SYS_DEFAULT:
                SubLangIDName = "SUBLANG_SYS_DEFAULT";
                break;
        }
    }

    if (LangIDName)
        printf("PRIMARYLANGID: 0x%02X (%s)\n", PRIMARYLANGID(LangID), LangIDName);
    else
        printf("PRIMARYLANGID: 0x%02X\n", PRIMARYLANGID(LangID));

    if (SubLangIDName)
        printf("SUBLANGID: 0x%02X (%s)\n", SUBLANGID(LangID), SubLangIDName);
    else
        printf("SUBLANGID: 0x%02X\n", SUBLANGID(LangID));
}

void check_sortid(LANGID LangID, WORD SortID)
{
    const char *Name = NULL;
    INT i, nCount = _countof(SortIDEntries);

    for (i = 0; i < nCount; ++i)
    {
        if (SortIDEntries[i].LangID != LangID)
            continue;
        if (SortIDEntries[i].SortID != SortID)
            continue;
        Name = SortIDEntries[i].Name;
        break;
    }
    if (Name == NULL)
    {
        if (SortID == SORT_DEFAULT)
            Name = "SORT_DEFAULT";
    }
    if (Name)
        printf("SORTID: 0x%04X (%s)\n", SortID, Name);
    else
        printf("SORTID: 0x%04X\n", SortID);
}

void check_lcid(LCID lcid)
{
    LANGID LangID;
    WORD PrimLangID, SortID;

    LangID = LANGIDFROMLCID(lcid);
    printf("LANGID: 0x%04X\n", LangID);

    check_langid(LangID);

    PrimLangID = PRIMARYLANGID(LangID);
    SortID = SORTIDFROMLCID(lcid);
    check_sortid(PrimLangID, SortID);
}

void check_machine(void)
{
    LCID lcid;
    LANGID LangID;

    LangID = GetUserDefaultLangID();
    printf("# User default LANGID: 0x%04X\n", LangID);
    check_langid(LangID);
    printf("---\n");

    LangID = GetSystemDefaultLangID();
    printf("# System default LANGID: 0x%04X\n", LangID);
    check_langid(LangID);
    printf("---\n");

    lcid = GetUserDefaultLCID();
    printf("# User default LCID: 0x%08lX\n", lcid);
    check_lcid(lcid);
    printf("---\n");

    lcid = GetSystemDefaultLCID();
    printf("# System default LCID: 0x%08lX\n", lcid);
    check_lcid(lcid);
    printf("---\n");

    lcid = GetThreadLocale();
    printf("# Thread LCID: 0x%08lX\n", lcid);
    check_lcid(lcid);
    printf("---\n");
}

int main(int argc, char **argv)
{
    if (argc <= 1)
    {
        check_machine();
        return 0;
    }

    if (strcmp(argv[1], "--langid") == 0 && argc >= 3)
    {
        LANGID LangID = strtoul(argv[2], NULL, 16);
        if (LangID == 0)
        {
            LangID = parse_langid(argv[2]);
        }
        if (LangID == 0xFFFF)
        {
            printf("ERROR: invalid LANGID\n");
            return 1;
        }
        check_langid(LangID);
        return 0;
    }

    if (strcmp(argv[1], "--lcid") == 0 && argc >= 3)
    {
        LCID lcid = strtoul(argv[2], NULL, 16);
        printf("LCID: 0x%08lX\n", lcid);
        check_lcid(lcid);
        return 0;
    }

    printf("ERROR: invalid argument\n");
    show_help();
    return -1;
}
